;; -*- coding: utf-8 -*-
;; Copyright 2008/2012 Vincent Belaïche

;; Author: Vincent Belaïche <vincent.b.1@hotmail.fr>
;; Version: $Id: accents-ascii.el,v 1.6 2013-07-27 10:54:31 Vincent Exp $
;; Keywords:
;; X-URL: not distributed yet

;;
;; This software is governed by the CeCILL license under French law and
;; abiding by the rules of distribution of free software.  You can  use,
;; modify and/ or redistribute the software under the terms of the CeCILL
;; license as circulated by CEA, CNRS and INRIA at the following URL
;; "http://www.cecill.info".
;;
;; As a counterpart to the access to the source code and  rights to copy,
;; modify and redistribute granted by the license, users are provided only
;; with a limited warranty  and the software's author,  the holder of the
;; economic rights,  and the successive licensors  have only  limited
;; liability.
;;
;; In this respect, the user's attention is drawn to the risks associated
;; with loading,  using,  modifying and/or developing or reproducing the
;; software by the user in light of its specific status of free software,
;; that may mean  that it is complicated to manipulate,  and  that  also
;; therefore means  that it is reserved for developers  and  experienced
;; professionals having in-depth computer knowledge. Users are therefore
;; encouraged to load and test the software's suitability as regards their
;; requirements in conditions enabling the security of their systems and/or
;; data to be ensured and,  more generally, to use and operate it in the
;; same conditions as regards security.
;;
;; The fact that you are presently reading this means that you have had
;; knowledge of the CeCILL license and that you accept its terms.
;;
;;; Commentary:

;; accents-tex : change les caractères accentués en accents tex
;; conserve les majuscules, sans demander confirmation.
;;
;; accents-de-tex : change les accents tex en caracteres accentués sans
;; demander confirmation.  les majuscules sont respectees.
;;

(require 'eieio)

(dolist (v '(accents-convertisseur-cache
	     accents-convertisseur-le-cache
	     accents-convertisseur-base
	     accents-convertisseur-simple
	     accents-convertisseur-inverse
	     accents-convertisseur-html
	     accents-convertisseur-de-html))
  (makunbound v))

(defclass accents-convertisseur-cache ()
  ((table-de-hashage
    :initarg :table-de-hashage)
   (expression-rationnelle
    :initarg :expression-rationnelle
    :documentation "Expression rationnelle s'accordant aux
    lettres accentuées à remplacer.")
   (id-convertisseur
    :initarg :id-convertisseur
    :initform nil
    :type symbol)
   (lecteur-clef
    :initarg :lecteur-clef
    :initform nil))
  "C'est la classe qui fait le boulot effectif de remplacement.")

(defvar accents-convertisseur-le-cache
  (make-instance 'accents-convertisseur-cache
		 :table-de-hashage (make-hash-table :test 'equal)))

(defmethod accents-conversion
  ((this accents-convertisseur-cache) &optional beg end)
  (let (cleanup)
    (save-excursion
      (goto-char (or beg (point-min)))
      (setq end (if (integerp end)
		    (let ((m (make-marker)))
		      (setq cleanup t)
		      (set-marker m end)
		      m)
		  end))
      (save-match-data
	(let ((case-fold-search nil)
	      (table-de-hashage (oref this :table-de-hashage))
	      (expression-rationnelle (oref this :expression-rationnelle))
	      (lecteur-clef (or (oref this :lecteur-clef)
				(function (lambda () (match-string-no-properties 0))))))

	  (while (re-search-forward expression-rationnelle end t)
	    (let ((replacement
		   (gethash  (funcall lecteur-clef)
			     table-de-hashage)))
	      (and replacement
		   (replace-match replacement t t)))))))
    (when cleanup
      (set-marker end nil))))

(defclass accents-convertisseur-base ()
  ((cache
    :allocation :class
    :documentation "État en cache du convertisseur"))
  "Classe de base pour un convertisseur"  )
(oset-default accents-convertisseur-base cache
	      accents-convertisseur-le-cache)

(defclass accents-convertisseur-simple
  (accents-convertisseur-base)
  ((id
    :initarg :id
    :documentation "Symbole d'itentification unique pour le
    cache.")
   (liste-de-remplacement
    :initarg :liste-de-remplacement
    :documentation "Liste d'association") )
  "")

(defmacro accents-definir-conversion
  (CLASS CAR CDR &optional LECTEUR-CLEF)
  `(defmethod accents-conversion
     ((this ,CLASS) &optional beg end)

     (let ((cache (oref this cache))
	   (id (oref this :id)))
       (unless (eq (oref cache :id-convertisseur)
		   id)
	 (oset cache :id-convertisseur ,LECTEUR-CLEF)
	 (oset cache :lecteur-clef nil)
	 (let ((table-de-hashage (oref cache :table-de-hashage))
	       (liste-de-remplacement (oref this :liste-de-remplacement)))
	   (clrhash table-de-hashage)
	   (dolist (k liste-de-remplacement)
	     (puthash (,CAR k) (,CDR k) table-de-hashage))
	   (oset cache :expression-rationnelle
		 (regexp-opt (mapcar (quote ,CAR) liste-de-remplacement)))))
       (accents-conversion cache beg end))))

(accents-definir-conversion accents-convertisseur-simple car cdr)

(defclass accents-convertisseur-inverse
  (accents-convertisseur-simple)
  ()
  "Convertisseur pour effectuer la conversion inverse vis-à-vis
  de la liste de remplacement.")

(accents-definir-conversion accents-convertisseur-inverse cdr car)

(defun accents-moteur (liste-des-remplacements &optional convert-to-re)
  (save-excursion
	(save-match-data
	  (let ((case-fold-search nil)
		(re (if convert-to-re
			(mapconcat 'identity
				   (mapcar 'car liste-des-remplacements)
				   "\\|")
		      (regexp-opt  (mapcar 'car liste-des-remplacements)))))
	    (goto-char (point-min))
	    (while (re-search-forward re nil t)
	      (replace-match
	       (if convert-to-re
		   (save-match-data
		     (assoc-default (match-string 0)
				    liste-des-remplacements 'string-match))
		 (cdr (assoc-string (match-string 0) liste-des-remplacements)))
	       t; fixed case
	       t; literal
	       ))))))

(defconst accents-convertisseur-tex
  (make-instance
   'accents-convertisseur-simple
   :id 'accents-convertisseur-tex
   :liste-de-remplacement
   '(("\Á" . "\\'A")
     ("\À" . "\\`A")
     ("\Â" . "\\^A")
     ("\Ä" .  "\\\"A")
     ("\É" . "\\'E")
     ("\È" . "\\`E")
     ("\Ê" . "\\^E")
     ("\Ë" .  "\\\"E")
     ("\Í" . "\\'I")
     ("\Ì" . "\\`I")
     ("\Î" . "\\^I")
     ("\Ï" .  "\\\"I")
     ("\0" . "\\'O")
     ("\Ò" . "\\`O")
     ("\Ô" . "\\^O")
     ("\Ö" .  "\\\"O")
     ("\Ú" . "\\'U")
     ("\Ù" . "\\`U")
     ("\Û" . "\\^U")
     ("\Ü" .  "\\\"U")
     ("\Ç" . "\\c{C}")
     ("\á" . "\\'a")
     ("\à" . "\\`a")
     ("\â" . "\\^a")
     ("\ä"  .  "\\\"a")
     ("\é" . "\\'e")
     ("\è" . "\\`e")
     ("\ê" . "\\^e")
     ("\ë" .  "\\\"e")
     ("\í" .  "\\'\\i")
     ("\ì" .  "\\`\\i")
     ("\î" .  "\\^\\i")
     ("\ï" .  "\\\"\\i")
     ("\ó" . "\\'o")
     ("\ò" . "\\`o")
     ("\ô" . "\\^o")
     ("\ö" .  "\\\"o")
     ("\ú" . "\\'u")
     ("\ù" . "\\`u")
     ("\û" . "\\^u")
     ("\ü" .  "\\\"u")
     ("\ç" . "\\c{c}")
     ("\×" . "\\times{}")
)

   )
  "Convertisseur de lettres accentuées en séquence TeX
  équivalentes.")

(defun accents-tex ()
  (interactive)
  "Change les caractères accentués en séquence TeX équivalentes."
  (let (beg end)
    (if (region-active-p)
	(setq beg (region-beginning)
	      end  (region-end)))
    (accents-conversion accents-convertisseur-tex beg end)
    ))

(defun accents-de-tex ()
  (interactive)
  "Change les les séquences TeX produisant des caractères
accentués en les caractères correspondants."
  (let (beg end)
    (if (region-active-p)
	(setq beg (region-beginning)
	      end  (region-end)))
    (accents-moteur
     '(( "\\\\'[ \n\t]*A" . "\Á")
       ( "\\\\`[ \n\t]*A" . "\À")
       ( "\\\\^[ \n\t]*A" . "\Â")
       ( "\\\\\"[ \n\t]*A" . "\Ä")
       ( "\\\\'[ \n\t]*E" . "\É")
       ( "\\\\`[ \n\t]*E" . "\È")
       ( "\\\\^[ \n\t]*E" . "\Ê" )
       ( "\\\\\"[ \n\t]*E" . "\Ë")
       ( "\\\\'[ \n\t]*I" . "\Í")
       ( "\\\\`[ \n\t]*I" . "\Ì")
       ( "\\\\^[ \n\t]*I" . "\Î")
       ( "\\\\\"[ \n\t]*I" . "\Ï")
       ( "\\\\'[ \n\t]*O" . "\0")
       ( "\\\\`[ \n\t]*O" . "\Ò")
       ( "\\\\^[ \n\t]*O" . "\Ô")
       ( "\\\\\"[ \n\t]*O" . "\Ö")
       ( "\\\\'[ \n\t]*U" . "\Ú")
       ( "\\\\`[ \n\t]*U" . "\Ù")
       ( "\\\\^[ \n\t]*U" . "\Û")
       ( "\\\\\"[ \n\t]*U" . "\Ü")
       ( "\\\\c[ \n\t]*{C}" . "\Ç")
       ( "\\\\'[ \n\t]*{A}" . "\Á")
       ( "\\\\`[ \n\t]*{A}" . "\À")
       ( "\\\\^[ \n\t]*{A}" . "\Â")
       ( "\\\\\"[ \n\t]*{A}" . "\Ä")
       ( "\\\\'[ \n\t]*{E}" . "\É")
       ( "\\\\`[ \n\t]*{E}" . "\È")
       ( "\\\\^[ \n\t]*{E}" . "\Ê" )
       ( "\\\\\"[ \n\t]*{E}" . "\Ë")
       ( "\\\\'[ \n\t]*{I}" . "\Í")
       ( "\\\\`[ \n\t]*{I}" . "\Ì")
       ( "\\\\^[ \n\t]*{I}" . "\Î")
       ( "\\\\\"[ \n\t]*{I}" . "\Ï")
       ( "\\\\'[ \n\t]*{O}" . "\0")
       ( "\\\\`[ \n\t]*{O}" . "\Ò")
       ( "\\\\^[ \n\t]*{O}" . "\Ô")
       ( "\\\\\"[ \n\t]*{O}" . "\Ö")
       ( "\\\\OE[ \n\t]*\\({}\\)?" . "\Œ")
       ( "\\\\'[ \n\t]*{U}" . "\Ú")
       ( "\\\\`[ \n\t]*{U}" . "\Ù")
       ( "\\\\^[ \n\t]*{U}" . "\Û")
       ( "\\\\'[ \n\t]*a" . "\á" )
       ( "\\\\`[ \n\t]*a" . "\à" )
       ( "\\\\^[ \n\t]*a" . "\â" )
       ( "\\\\\"[ \n\t]*a" . "\ä" )
       ( "\\\\'[ \n\t]*e" . "\é" )
       ( "\\\\`[ \n\t]*e" . "\è")
       ( "\\\\^[ \n\t]*e" . "\ê")
       ( "\\\\\"[ \n\t]*e" . "\ë")
       ( "\\\\'{\\\\i}" . "\í")
       ( "\\\\`{\\\\i}" . "\ì")
       ( "\\\\^{\\\\i}" . "\î")
       ( "\\\\\"{\\\\i}" . "\ï")
       ( "\\\\'\\\\[ \n\t]*i" . "\í")
       ( "\\\\`\\\\[ \n\t]*i" . "\ì")
       ( "\\\\^\\\\[ \n\t]*i" . "\î")
       ( "\\\\\"\\\\[ \n\t]*i" . "\ï")
       ( "\\\\'[ \n\t]*o" . "\ó")
       ( "\\\\`[ \n\t]*o" . "\ò")
       ( "\\\\^[ \n\t]*o" . "\ô")
       ( "\\\\\"[ \n\t]*o" . "\ö")
       ( "\\\\oe[ \n\t]*\\({}\\)?" . "\œ")
       (  "\\\\'[ \n\t]*u" . "\ú")
       ( "\\\\`[ \n\t]*u" . "\ù")
       ( "\\\\^[ \n\t]*u" . "\û")
       ( "\\\\\"[ \n\t]*u" . "\ü")
       ( "\\\\c[ \n\t]*{c}" . "\ç")
       ( "\\\\\"[ \n\t]*{u}" . "\ü")
       ( "\\\\'[ \n\t]*{a}" . "\á" )
       ( "\\\\`[ \n\t]*{a}" . "\à" )
       ( "\\\\^[ \n\t]*{a}" . "\â" )
       ( "\\\\\"[ \n\t]*{a}" . "\ä" )
       ( "\\\\'[ \n\t]*{e}" . "\é" )
       ( "\\\\`[ \n\t]*{e}" . "\è")
       ( "\\\\^[ \n\t]*{e}" . "\ê")
       ( "\\\\\"[ \n\t]*{e}" . "\ë")
       ( "\\\\'[ \n\t]*{o}" . "\ó")
       ( "\\\\`[ \n\t]*{o}" . "\ò")
       ( "\\\\^[ \n\t]*{o}" . "\ô")
       ( "\\\\\"[ \n\t]*{o}" . "\ö")
       (  "\\\\'[ \n\t]*{u}" . "\ú")
       ( "\\\\`[ \n\t]*{u}" . "\ù")
       ( "\\\\^[ \n\t]*{u}" . "\û")
       ( "\\\\\"[ \n\t]*{u}" . "\ü")
       )
     t ;; utilisation de d'expression régulière
     )
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; accents-texinfo : change les caractères accentués en accents texinfo sans
;; demander confirmation les majuscules sont respectées
;;
;; texinfo-accents : change les accents Texinfo en caractères accentués sans
;; demander confirmation les majuscules sont respectées

(defconst accents-convertisseur-texinfo
  (make-instance
   'accents-convertisseur-simple
   :id 'accents-convertisseur-texinfo
   :liste-de-remplacement
   '(("\Á" . "@'A")
     ("\À" . "@`A")
     ("\Â" . "@^A")
     ("\Ä" .  "@\"A")
     ("\É" . "@'E")
     ("\È" . "@`E")
     ("\Ê" . "@^E")
     ("\Ë" .  "@\"E")
     ("\Í" . "@'I")
     ("\Ì" . "@`I")
     ("\Î" . "@^I")
     ("\Ï" .  "@\"I")
     ( "\Œ" . "@OE{}")
     ("\0" . "@'O")
     ("\Ç" . "@,{C}")
     ("\Ò" . "@`O")
     ("\Ô" . "@^O")
     ("\Ö" .  "@\"O")
     ("\Ø" . "@O{}")
     ("\¿" . "@questiondown{}")
     ("\¡" . "@exclamdown{}")
     ("\Ù" . "@`U")
     ("\Ú" . "@'U")
     ("\Û" . "@^U")
     ("\Ü" .  "@\"U")
     ("\ß" . "@ss{}")
     ("\æ" . "@ae{}")
     ( "\–" . " -- ")
     ( "\—" . " --- ")
     ("\á" . "@'a")
     ("\à" . "@`a")
     ("\â" . "@^a")
     ("\ä"  .  "@\"a")
     ("\é" . "@'e")
     ("\è" . "@`e")
     ("\ê" . "@^e")
     ("\ë" .  "@\"e")
     ("\í" .  "@'{@dotless{i}}")
     ("\ì" .  "@`{@dotless{i}}")
     ("\î" .  "@^{@dotless{i}}")
     ("\ï" .  "@\"{@dotless{i}}")
     ("\ó" . "@'o")
     ("\ò" . "@`o")
     ("\ø" . "@o{}")
     ( "\œ" . "@oe{}")
     ("\ô" . "@^o")
     ("\ö" .  "@\"o")
     ("\ú" . "@'u")
     ("\ù" . "@`u")
     ("\û" . "@^u")
     ("\ü" .  "@\"u")
     ("\ç" . "@,{c}")
     ("\ " . "@tie{}")     ))
  "Convertisseur de lettres accentuées en séquence Texinfo
  équivalentes.")

(defconst accents-convertisseur-de-texinfo
  (make-instance
   'accents-convertisseur-inverse
   :id 'accents-convertisseur-de-texinfo
   :liste-de-remplacement (oref accents-convertisseur-texinfo
				:liste-de-remplacement))
  "Convertisseur d'entités Texinfo en lettres accentuées équivalentes" )

(defun accents-texinfo ()
  (interactive)
  "change les caractères accentués en entités Texinfo équivalentes."
  (let (beg end)
    (if (region-active-p)
	(setq beg (region-beginning)
	      end  (region-end)))
    (accents-conversion accents-convertisseur-texinfo beg end)))


(defun accents-de-texinfo ()
  (interactive)
  "change les entités TEXINFO en caractères accentués équivalents."
  (let (beg end)
    (if (region-active-p)
	(setq beg (region-beginning)
	      end  (region-end)))
    (accents-conversion accents-convertisseur-de-texinfo beg end)))


(defconst accents-convertisseur-quote-texinfo
  (make-instance
   'accents-convertisseur-simple
   :id 'accents-convertisseur-quote-texinfo
   :liste-de-remplacement
   '(("@" . "@@" )
     ("{" . "@{")
     ("}" . "@}")   ))
  "Convertisseur de caractères spéciaux Texinfo en leurs
  séquences d'échappement respectives.")

(defconst accents-convertisseur-de-quote-texinfo
  (make-instance
   'accents-convertisseur-inverse
   :id 'accents-convertisseur-de-quote-texinfo
   :liste-de-remplacement (oref accents-convertisseur-quote-texinfo
				:liste-de-remplacement))
  "Convertisseur de caractères spéciaux échappés Texinfo en caractères non échappés." )

(defun accents-quote-texinfo ()
  (interactive)
  "Change les caractère spéciaux TEXINFO non échappés `@', `{' et
`}' en caractères échappés `@@', `@{' et `@}'."

  (let (beg end)
    (if (region-active-p)
	(setq beg (region-beginning)
	      end  (region-end)))
    (accents-conversion accents-convertisseur-quote-texinfo beg end)))


(defun accents-de-quote-texinfo ()
  (interactive)
  "Change les caractère spéciaux TEXINFO échappés `@@', `@{' et
`@}' en caractères non échappés `@', `{' et `}'."
  (let (beg end)
    (if (region-active-p)
	(setq beg (region-beginning)
	      end  (region-end)))
    (accents-conversion accents-convertisseur-de-quote-texinfo beg end)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; accents-html : change les caractères accentués en accents HTML sans demander
;; confirmation les majuscules sont respectées
;;
;; html-accents : change les accents HTML en caractères accentués sans demander
;; confirmation les majuscules sont respectées

(defconst accents-convertisseur-html
  (make-instance 'accents-convertisseur-simple
		 :id 'accents-convertisseur-html
		 :liste-de-remplacement
   '(
     ( "\∀" . "&forall;")
     ( "\∂" . "&part;")
     ( "\∃" . "&exist;")
     ( "\∅" . "&empty;")
     ( "\∇" . "&nabla;")
     ( "\∈" . "&isin;")
     ( "\∉" . "&notin;")
     ( "\∋" . "&ni;")
     ( "\∏" . "&prod;")
     ( "\∑" . "&sum;")
     ( "\−" . "&minus;")
     ( "\∗" . "&lowast;")
     ( "\√" . "&radic;")
     ( "\∝" . "&prop;")
     ( "\∞" . "&infin;")
     ( "\∠" . "&ang;")
     ( "\∧" . "&and;")
     ( "\∨" . "&or;")
     ( "\∩" . "&cap;")
     ( "\∪" . "&cup;")
     ( "\∫" . "&int;")
     ( "\∴" . "&there4;")
     ( "\∼" . "&sim;")
     ( "\≅" . "&cong;")
     ( "\≈" . "&asymp;")
     ( "\≠" . "&ne;")
     ( "\≡" . "&equiv;")
     ( "\≤" . "&le;")
     ( "\≥" . "&ge;")
     ( "\⊂" . "&sub;")
     ( "\⊃" . "&sup;")
     ( "\⊄" . "&nsub;")
     ( "\⊆" . "&sube;")
     ( "\⊇" . "&supe;")
     ( "\⊕" . "&oplus;")
     ( "\⊗" . "&otimes;")
     ( "\⊥" . "&perp;")
     ( "\⋅" . "&sdot;")
     ( "\Α" . "&Alpha;")
     ( "\Β" . "&Beta;")
     ( "\Γ" . "&Gamma;")
     ( "\Δ" . "&Delta;")
     ( "\Ε" . "&Epsilon;")
     ( "\Ζ" . "&Zeta;")
     ( "\Η" . "&Eta;")
     ( "\Θ" . "&Theta;")
     ( "\Ι" . "&Iota;")
     ( "\Κ" . "&Kappa;")
     ( "\Λ" . "&Lambda;")
     ( "\Μ" . "&Mu;")
     ( "\Ν" . "&Nu;")
     ( "\Ξ" . "&Xi;")
     ( "\Ο" . "&Omicron;")
     ( "\Π" . "&Pi;")
     ( "\Ρ" . "&Rho;")
     ( "\Σ" . "&Sigma;")
     ( "\Τ" . "&Tau;")
     ( "\Υ" . "&Upsilon;")
     ( "\Φ" . "&Phi;")
     ( "\Χ" . "&Chi;")
     ( "\Ψ" . "&Psi;")
     ( "\Ω" . "&Omega;")
     ( "\α" . "&alpha;")
     ( "\β" . "&beta;")
     ( "\γ" . "&gamma;")
     ( "\δ" . "&delta;")
     ( "\ε" . "&epsilon;")
     ( "\ζ" . "&zeta;")
     ( "\η" . "&eta;")
     ( "\θ" . "&theta;")
     ( "\ι" . "&iota;")
     ( "\κ" . "&kappa;")
     ( "\λ" . "&lambda;")
     ( "\μ" . "&mu;")
     ( "\ν" . "&nu;")
     ( "\ξ" . "&xi;")
     ( "\ο" . "&omicron;")
     ( "\π" . "&pi;")
     ( "\ρ" . "&rho;")
     ( "\ς" . "&sigmaf;")
     ( "\σ" . "&sigma;")
     ( "\τ" . "&tau;")
     ( "\υ" . "&upsilon;")
     ( "\φ" . "&phi;")
     ( "\χ" . "&chi;")
     ( "\ψ" . "&psi;")
     ( "\ω" . "&omega;")
     ( "\ϑ" . "&thetasym;")
     ( "\ϒ" . "&upsih;")
     ( "\ϖ" . "&piv;")
     ( "\Œ" . "&OElig;")
     ( "\œ" . "&oelig;")
     ( "\Š" . "&Scaron;")
     ( "\š" . "&scaron;")
     ( "\Ÿ" . "&Yuml;")
     ( "\ƒ" . "&fnof;")
     ( "\ˆ" . "&circ;")
     ( "\˜" . "&tilde;")
     ( "\ " . "&ensp;")
     ( "\ " . "&emsp;")
     ( "\ " . "&thinsp;")
     ( "\‌" . "&zwnj;")
     ( "\‍" . "&zwj;")
     ( "\‎" . "&lrm;")
     ( "\‏" . "&rlm;")
     ( "\–" . "&ndash;")
     ( "\—" . "&mdash;")
     ( "\‘" . "&lsquo;")
     ( "\’" . "&rsquo;")
     ( "\‚" . "&sbquo;")
     ( "\“" . "&ldquo;")
     ( "\”" . "&rdquo;")
     ( "\„" . "&bdquo;")
     ( "\†" . "&dagger;")
     ( "\‡" . "&Dagger;")
     ( "\•" . "&bull;")
     ( "\…" . "&hellip;")
     ( "\‰" . "&permil;")
     ( "\′" . "&prime;")
     ( "\″" . "&Prime;")
     ( "\‹" . "&lsaquo;")
     ( "\›" . "&rsaquo;")
     ( "\‾" . "&oline;")
     ( "\¢" . "&cent;")
     ( "\€" . "&euro;")
     ( "\£" . "&pound;")
     ( "\¥" . "&yen;")
     ( "\§" . "&sect;")
     ( "\©" . "&copy;")
     ( "\®" . "&reg;")
     ( "\™" . "&trade;")
     ( "\ℵ" . "&alefsym;")
     ( "\←" . "&larr;")
     ( "\↑" . "&uarr;")
     ( "\→" . "&rarr;")
     ( "\↓" . "&darr;")
     ( "\↔" . "&harr;")
     ( "\↵" . "&crarr;")
     ( "\⇐" . "&lArr;")
     ( "\⇒" . "&rArr;")
     ( "\⇔" . "&hArr;")
     ( "\⇓" . "&dArr;")
     ( "\⇑" . "&uArr;")
     ( "\↨"  . "&#8616;")
     ( "\⌈" . "&lceil;")
     ( "\⌉" . "&rceil;")
     ( "\⌊" . "&lfloor;")
     ( "\⌋" . "&rfloor;")
     ( "\◊" . "&loz;")
     ( "\♠" . "&spades;")
     ( "\♣" . "&clubs;")
     ( "\♥" . "&hearts;")
     ( "\♦" . "&diams;")
     ( "\Á" . "&Aacute;")
     ( "\À" . "&Agrave;")
     ( "\Â" . "&Acirc;")
     ( "\Ä" . "&Auml;")
     ( "\É" . "&Eacute;")
     ( "È" . "&Egrave;")
     ( "\Ê" . "&Ecirc;")
     ( "\Ë" . "&Euml;")
     ( "\Í" . "&Iacute;")
     ( "\Ì" . "&Igrave;")
     ( "\Î" . "&Icirc;")
     ( "\Ï" . "&Iuml;")
     ( "\Ó" . "&Oacute;")
     ( "\Ò" . "&Ograve;")
     ( "\Ô" . "&Ocirc;")
     ( "\Ö" . "&Ouml;")
     ( "\Ú" . "&Uacute")
     ( "\Ù" . "&Ugrave;")
     ( "\Û" . "&Ucirc;")
     ( "\Ü" . "&Uuml;")
     ( "\Ç" . "&Ccedil;")
     ( "\á" . "&aacute;")
     ( "\à" . "&agrave;")
     ( "\â" . "&acirc;")
     ( "\ä" . "&auml;")
     ( "\é" . "&eacute;")
     ( "\è" . "&egrave;")
     ( "\ê" . "&ecirc;")
     ( "\ë" . "&euml;")
     ( "\…" . "&hellip;")
     ( "\í" . "&iacute;")
     ( "\ì" . "&igrave;")
     ( "\î" . "&icirc;")
     ( "\ "  . "&nbsp;")
     ( "\ï" . "&iuml;")
     ( "\ó" . "&oacute;")
     ( "\ò" . "&ograve;")
     ( "\œ" . "&oelig;")
     ( "\ô" . "&ocirc;")
     ( "\ö" . "&ouml;")
     ( "\ú" . "&uacute")
     ( "\ù" . "&ugrave;")
     ( "\û" . "&ucirc;")
     ( "\ü" . "&uuml;")
     ( "\ç" . "&ccedil;")
     ("\«"  . "&laquo;")
     ("\»"  . "&raquo;")
     ("\ß" . "&szlig;")
     ("\Þ" . "&THORN;")
     ("" . "&#357;")
     ("\ž" . "&#382;")
     ("\¡" . "&iexcl;")
     ("\¤" . "&curren;")
     ("\¦" . "&brvbar;")
     ("\¨" . "&uml;")
     ("\ª" . "&ordf;")
     ("\¬" . "&not;")
     ("\­" . "&shy;")
     ("\¯" . "&macr;")
     ("\°" . "&deg;")
     ("\±" . "&plusmn;")
     ("\²" . "&sup2;")
     ("\³" . "&sup3;")
     ("\´" . "&acute;")
     ("\µ" . "&micro;")
     ("\¶" . "&para'")
     ("\·" . "&middot;")
     ("\¸" . "&cedil;")
     ("\¹" . "&sup1;")
     ("\º" . "&ordm;")
     ("\¼" . "&frac14;")
     ("\½" . "&frac12;")
     ("\¾" . "&frac34;")
     ("\¿" . "&iquest;")
     ("\Ã" . "&Atilde;")
     ("\Å" . "&Aring;")
     ("\Æ" . "&AElig;")
     ("\Ð" . "&ETH;")
     ("\Ñ" . "&Ntilde;")
     ("\×" . "&times;")
     ("\Ø" . "&Oslash;")
     ("\Ý" . "&Yacute;")
     ("\ã" . "&atilde;")
     ("\å" . "&aring;")
     ("\æ" . "&aelig;")
     ("\ð" . "&eth;")
     ("\ñ" . "&ntilde;")
     ("\õ" . "&otilde;")
     ("\÷" . "&divide;")
     ("\ø" . "&oslash;")
     ("\ý" . "&yacute;")
     ("\þ" . "&thorn;")
     ("\ÿ" . "&yuml;")
     ) )
  "Convertisseur de lettres accentuées en entités HTML équivalentes.
Cf http://www.webdesigneuse.net/Trouver-une-entite-HTML.html")

(defconst accents-convertisseur-de-html
  (make-instance
   'accents-convertisseur-inverse
   :id 'accents-convertisseur-de-html
   :liste-de-remplacement (oref accents-convertisseur-html
				:liste-de-remplacement))
  "Convertisseur d'entités HTML en lettres accentuées équivalentes" )

(defun accents-html ()
  (interactive)
  "change les caractères accentués en entités HTML équivalentes."
  (let (beg end)
    (if (region-active-p)
	(setq beg (region-beginning)
	      end  (region-end)))
    (accents-conversion accents-convertisseur-html beg end)))


(defun accents-de-html ()
  (interactive)
  "change les entités HTML en caractères accentués équivalents."
  (let (beg end)
    (if (region-active-p)
	(setq beg (region-beginning)
	      end (let ((m (make-marker))) (set-marker m (region-end)) m)))
    (accents-conversion accents-convertisseur-de-html beg end)
    (save-excursion
      (save-match-data
	(goto-char (or beg (point-min)))
	(while (re-search-forward "&#\\([0-9]+\\);" end t)
	  (replace-match (string (string-to-number (match-string 1))) t t))))
    (when end (set-marker end nil))))

;;---------------------------------------------------------------------------

(defconst accents-convertisseur-quote-html
  (make-instance 'accents-convertisseur-simple
		 :id 'accents-convertisseur-quote-html
		 :liste-de-remplacement
		 '( ( "<" . "&lt;")
		    ( ">" . "&gt;")
		    ( "\"" . "&quot;")))
  "Convertisseur des caractères `\"', `<', et `>' en entités HTML
  &quot; &lt; et &gt correspondantes.")

(defconst accents-convertisseur-de-quote-html
  (make-instance
   'accents-convertisseur-inverse
   :id 'accents-convertisseur-de-quote-html
   :liste-de-remplacement (oref accents-convertisseur-quote-html
				:liste-de-remplacement))
  "Convertisseur des entités HTML &quot; &lt; et &gt, en caractères correspondants." )

(defun accents-quote-html ()
  (interactive)
  "Convertisseur des caractères `\"', `<', et `>' en entités HTML
  &quot; &lt; et &gt correspondantes."
  (let (beg end)
    (if (region-active-p)
	(setq beg (region-beginning)
	      end  (region-end)))
    (accents-conversion accents-convertisseur-quote-html beg end)))

(defun accents-region-default-whole-buffer-interactive ()
  "Specification d'interaction pour les fonctions
`accents-de-utf-8' et `accents-utf-8'."
  (if (region-active-p)
    (list (region-beginning) (region-end))
    (list (point-min) (point-max))))

(defun accents-de-utf-8 (beg end)
  (interactive (accents-region-default-whole-buffer-interactive))
  "Change les séquences UTF-8 en caractères accentués équivalents."
  (save-excursion
    (let ((s (apply 'unibyte-string (mapcar 'identity
					    (buffer-substring-no-properties beg end)))))
      (save-excursion
	(goto-char beg)
	(delete-region beg end)
	(setq s (decode-coding-string s 'utf-8 t (current-buffer)))))))

(defun accents-utf-8 (beg end)
  (interactive (accents-region-default-whole-buffer-interactive))
  "Change les caractères accentués en séquences UTF-8 équivalentes."
  (let ((s (buffer-substring beg end)))
    (save-excursion
      (goto-char beg)
      (delete-region beg end)
      (encode-coding-string s 'utf-8 t (current-buffer)))))

(provide 'accents-ascii)
